@@ -3,9 +3,11 @@ Requires simplejson; can be downloaded from
 http://cheeseshop.python.org/pypi/simplejson
 """
 import xmlrpclib
+from datetime import datetime
 
 import simplejson
 
+
 # From xmlrpclib
 SERVER_ERROR          = xmlrpclib.SERVER_ERROR
 NOT_WELLFORMED_ERROR  = xmlrpclib.NOT_WELLFORMED_ERROR
@@ -18,20 +20,33 @@ INTERNAL_ERROR        = xmlrpclib.INTERNAL_ERROR
 # Custom errors
 METHOD_NOT_CALLABLE   = -32604
 
+
 class Fault(xmlrpclib.Fault):
     pass
 
+
 class NoSuchFunction(Fault):
     """
     There is no function by the given name.
     """
 
+class JSONRPCEncoder(simplejson.JSONEncoder):
+    """
+    Provide custom serializers for JSON-RPC.
+    """
+    def default(self, obj):
+        if isinstance(obj, datetime):
+            return obj.strftime("%Y%m%dT%H:%M:%S")
+        raise TypeError("%r is not JSON serializable" % (obj,))
+
+
 def dumps(obj, **kws):
     if isinstance(obj, Exception):
         obj = {'fault': obj.__class__.__name__,
             'faultCode': obj.faultCode,
             'faultString': obj.faultString}
-    return simplejson.dumps(obj, **kws)
+    return simplejson.dumps(obj, cls=JSONRPCEncoder, **kws)
+
 
 def loads(string, **kws):
     unmarshalled = simplejson.loads(string, **kws)
@@ -41,13 +56,17 @@ def loads(string, **kws):
             unmarshalled['faultString'])
     return unmarshalled
 
+
 class SimpleParser(object):
 
+    buffer = ''
+
     def feed(self, data):
-        self.data = loads(data)
+        self.buffer += data
 
     def close(self):
-        pass
+        self.data = loads(self.buffer)
+
 
 class SimpleUnmarshaller(object):
 
@@ -59,12 +78,14 @@ class SimpleUnmarshaller(object):
             return self.parser.data.get("params")
         return self.parser.data
 
+
 def getparser():
     parser = SimpleParser()
     marshaller = SimpleUnmarshaller()
     marshaller.parser = parser
     return parser, marshaller
 
+
 class Transport(xmlrpclib.Transport):
     """
     Handles an HTTP transaction to an XML-RPC server.
@@ -77,6 +98,7 @@ class Transport(xmlrpclib.Transport):
         """
         return getparser()
 
+
 class ServerProxy(xmlrpclib.ServerProxy):
     """
 
